;ut62_from_source_herb.lst
;originally based on UT71 source +ut62 ROM disasm for A18 syntax 2017 
;hand edited from UT62 listing from MPM-217, imaged by Ed Keefe July 2020
;last edit July 10 2020 HRJ
;
; register equivalents for A18
R0              EQU     0
R1              EQU     1
R2              EQU     2
R3              EQU     3
R4              EQU     4
R5              EQU     5
R6              EQU     6
R7              EQU     7
R8              EQU     8
R9              EQU     9
RA              EQU     10
RB              EQU     11
RC              EQU     12
RD              EQU     13
RE              EQU     14
RF              EQU     15

;*****************************************************
; NAME: UT62/4
; DESC : RCA MICROBOARD MONITOR FOR CASSETTE I/O
; DATE: 6/16/82
;*****************************************************


;UT62 IS A MONITOR PROGRAM DESIGNED TO
;EXAMINE MEMORY, ALTER MEMORY, AND TO
;TO BEGIN PROGRAM EXECUTION AT A GIVEN
;LOCATION. ALSO, PROGRAMS CAN BE SAVED ON TAPE. THESE FUNCTIONS
;ARE ACCOMPLISHED THROUGH A SERIES OF MONITOR
;COMMANDS WHICH
;ARE INITIATED BY TYPING A,B,C,D,E,F,I,J,M,P,R,S,OR W.
;THE FUNCTIONS INCLUDE MEMORY INSERT [I],
;MEMORY DISPLAY [D], MEMORY MOVE[M],
;MEMORY FILL[F], MEMORY SUBSTITUTE[S] ,READ TAPE[R],
;RUN EDITOR[E],RUN ASSEMBLER[A],RUN BASIC[B],
;WRITE TAPE[W] ,REWIND TAPES[J] ,COPY TAPE TO SCREEN [C],
;AND RUN PROGRAM[P].
;TAPE READ LOADS MEMORY FROM TAPE.
;TAPE WRITE SAVES MEMORY ON TAPE FROM START TO
;END ADDRESS AS SPECIFIED.
;ALSO INCLUDED ARE THE STANDARD READ AND
;TYPE ROUTINES WHICH PROVIDE COMMUNICATION
;WITH THE USERS TERMINAL.
;UPON INVOCATION, THE CONTENTS OF THE
;CPU REGISTERS ARE SAVED IN RAM~ AT #8COO.
;THE CONTENTS OF RO, R1, AND R4. 1 ARE
;DESTROYED, HOWEVER, BY THE PROCESS.
;THEIR CONTENTS CAN BE EXAMINED BY DOING
;D8000 FOR 20 BYTES.

;***********************************************
;     SYSTEM EQUATES, CONSTANTS &OFFSETS
;***********************************************
;
;
;     REGISTER ASSIGNMENTS - GEN. & UTILITIES

SP     EQU 002H ;STACK POINTER
PC     EQU 003H ;PROGRAM COUNTER
CALL   EQU 004H ;CALL ROUTINE REGISTER
RETN   EQU 005H ;RETURN ROUTINE REGISTER
LINK   EQU 006H ;SUBROUTINE.DATA LINK
BYTSTR EQU 07 ;TEMPORARY STORAGE DURING WRITE
SRTADD EQU 07 ;START ADDRESS FOR TAPE WRITE
PARITY EQU 07 ;FOR TAPE ROUTINES
TMPRG1 EQU 07 ;TEMPORARY REGISTER
BITRDR EQU 08 ;POINTS TO THE ROUTINE TO
BITWRT EQU 08 ;POINTS TO ROUTINE TO WRITE A BIT TO TAPE
ENDADD EQU 08 ;HOLDS END ADDRESS FOR WRITE TO TAPE
ADRPTR EQU 008H ;HOLDS ADDR DURING STORE FROM TP
TMPRG2 EQU 008H ;TEMPORARY REGISTER
LDRCNT EQU 009 ;COUNTER FOR TAPE LEADER
BITCNT EQU 009 ; COUNTER FOR BITS PER WORD FROM TAPE   
TMPRG3 EQU 009H ;TEMPORARY REGISTER
BUFPTR EQU 00AH ;POINTS TO TAPE READ/WRITE BUFFERS
CNT    EQU 00AH ;BYTE COUNT
SRC    EQU 00BH ;SOURCE REGISTER
TPTR   EQU 00BH ;TABLE POINTER
BLKFLG EQU 00BH ;FLAC TO SIGNAL TO WRITE A BLOCK
DELAY  EQU 00CH ;DELAY ROUTINE COUNTER
PTER   EQU 00CH ; IOCB PTR
PTR    EQU 00CH ;IOCB PTR
ASL    EQU 00DH ; HEX INPUT REGISTER 
DEST   EQU 00DH ;DESTINATION REGISTER
AUX    EQU 00EH ;AUX.1 HOLDS BIT TIME CONSTANT
CHAR   EQU 00FH ; STORES ASCII I/O
FLAG   EQU 00FH ; FLAGS FOR TAPE

;     RAM / ROM ALLOCATIONS

RDBUFF EQU 08E7FH
TOPSTK EQU 008CFFH
WRAM   EQU 08C1FH
WRTBUF EQU 08E80H
UT62   EQU 08000H

; ROM BASED PROGRAM HIGH ORDER ADDRESS VALUES

ABAS EQU 0B0H
AEDIT EQU 090H
AASM EQU 09BH
PRMPGR EQU 0E0H

;     ASCII CONTROL CHARACTERS

NULL   EQU 000H ;NULL
COMMA  EQU 2CH ;COMMA
SEMCOL EQU 3BH ;SEMICOLON
BS     EQU 008H ;BACK SPACE
LF     EQU 00AH ;LINE FEED
CR     EQU 00DH ;CARRIAGE RETURN
EOF    EQU 013H ;END OF FILE
SPACE  EQU 020H ;SPACE
CRLF   EQU 00D0AH ;CR LF

; CONSTANTS
BDSEL  EQU 001H ;PORT FOR TWO LEVEL I/O SELECT
BTHDRV EQU 0C0H ;SELECTS BOTH TAPE DRIVES
DRIVE0 EQU 040H ;SELECTS TAPE DRIVE 0
DRIVE1 EQU 080H ;SELECTS TAPE DRIVE 1
HIGHT   EQU 01H ;WRITE A HIGH TO TAPE
LNECNT EQU 00FH ;# OF BYTES PER LINE IN DISPLAY ROUTINE
LINES  EQU 014H ;NUMBER OF LINES PER SCREEN LOAD
LOWT    EQU 00H ;WRITE A LOW TO TAPE
PGMSRT EQU 00005H ;START ADDRESS FOR INIT1
PROMPT EQU 02AH ;PROMPT CHARACTER
TAPEIO EQU 02 ;SELECTS TAPE RECORDER
TPEOFF EQU 00 ;TURN OFF TAPES
TPESEL EQU 04 ;PORT TO SELECT TAPES
TPEWRT EQU 05 ;PORT TO WRITE TO TAPE
TRMINL EQU 01H ;SELECTS THE TERMINAL 


;*************************************************************************
	ORG UT62

	DIS
	DB 0                   ;DISABLE
	LDI HIGH(UT62) 		 ;ESTABLISH PROGRAM COUNTER AT
	PHI	R0               ;8000 HEX

;*************************************************************************
; REGISTER SAVE
; SAVES CONTENTS OF THE CPU REGISTERS @ 8C00

	LDI 	HIGH(WRAM-1)
	PHI	R1      	 
	LDI	LOW(WRAM-1)
	PLO	R1
	LDI	0A0H		;800B-F8 A0
	PHI	R4		;800D-B4
	SEX 	R1
	
LOOP	LDI	0D0H		;800F-F8 D0
	STR	R1		;8011-51
	XOR			;8012-F3
	BNZ	UT62A       ;LEAVE IF NO RAM THERE
	DEC	R1		;8015-21
	GHI	R4		;8016-94
	ADI	070H		;8017-FC 70 >p<
	BDF	LOOP1		;8019-33 1D
	ADI	021H		;801B-FC 21
LOOP1 ADI	07FH		;801D-FC 7F
	PHI	R4		;801F-B4
	STR	R1		;8020-51
	SEP	R1		;8021-D1
	STXD			;8022-73
	DEC	R1		;8023-21
	GHI	R4		;8024-94
	XRI	091H		;8025-FB 91
	BNZ	LOOP		;8027-3A 0F

UT62A	LBR	INIT		;8029


START				;802C
	SEP	CALL
	DW	TIMALC

;*************************************************************************
; OUTPUT THE UTILITY PROMPT
;*************************************************************************

PRMPT				;802F
	SEP	CALL
	DW	TPOFF
	SEP	CALL
	DW	OSTRNG          ;TYPE SCAN MODE '*' PROMPT
	DW 	CRLF
	DB 	PROMPT
	DB 	0

;                     MONITOR COMMAND INTERPRETER
; FETCHES THE ADDRESS FRCH THE COMMAND TABLE AND SETS
; THE PROGRAM COUNTER TO IT
; REG USED: PTR. CHAR. SP, ASL

 	LDI	LOW(TAB2)	;8039-F8 5A >Z<
	PLO	TPTR		;803B-AB
	LDI	HIGH(TAB2)		;803C-F8 80
	PHI	TPTR		;803E-BB
SCNLTR 	SEP	CALL		;803F-D4 CALL+addr
	DW	READ		;813EH
	SEX  TPTR

SCAN	GHI	CHAR		;8043-9F
	XOR			;8044-F3
	INC	TPTR		;8045-1B
	BZ	GOTHRE		;8046-32 4F
	INC	TPTR		;8048-1B
	INC	TPTR		;8049-1B
	LDN	TPTR		;804A-0B
	BZ	ERROR		;804B-32 85
	BR	SCAN		;804D-30 43
GOTHRE	DEC	SP		;804F-22
	DEC	SP		;8050-22
	LDA	TPTR		;8051-4B
R8052	PHI	LINK		;8052-B6
	LDA	TPTR		;8053-4B
	PLO	LINK		;8054-A6
	LDI	000H		;8055-F8 00
	PHI	ASL		;8057-BD
	PLO	ASL		;8058-AD
	SEP	RETN		;8059-D5 RET

TAB2  DB 'D'
      DW  (DISPLY) ;MEMORY DISPLAY
      DB 'I'
      DW  (INSERT);INSERT INTO MEMORY
      DB 'M'
      DW  (MOVE) ;MOVE A BLOCK OF MEMORY
      DB 'F'
      DW  (FILL); FILL A BLOCK OF MEMORY
      DB 'S'
      DW  (SUBST); BYTE SUBSTITUTION
      DB 'P'
      DW  (RUN); RUN A USER PROGRAM
      DB 'R' ; READ FROM TAPE TO MEMORY
      DW TPLOAD
	 DB 'J' ;REWIND TAPES
	 DW TPWIND
      DB 'W'
      DW  TPWRIT ;TAPE WRITE  
	 DB 'E'
	 DW EDITOR ;START THE EDITOR 
	 DB 'A'
	 DW ASSEMB
	 DB 'B'
	 DW BASIC
	 DB 'Z'
	 DW PROMIT ;START CPU AT E000H, P&X = 0
	 DB 'C'
	 DW TPSCRN ;COPY FROM TAPE TO SCREEN 
      DB 0
; UTILITY ERROR MESSAGE
R8085
ERROR LDI LOW(TOPSTK)
	PLO SP
	LDI HIGH(TOPSTK)
	PHI SP
	SEP CALL
	DW (OSTRNG)	;83F0H
R808E
	DW (CRLF)
	DB 'ERROR'

	DB 0
R8096
PRMPT1	LBR RENTER

;*************************************************************************
;                  START OF SUBROUTINES
;*************************************************************************
; MONITOR SUBSTITUTE FUNCTION
SUBST  SEP CALL
       DW (READHX)      ;87EA      ;INPUT ADDRESS
	GHI ASL
	PHI SRC        ;SAVE START ADDRESS
	GLO ASL
	PLO SRC

DECODE GHI CHAR
	XRI LF        ;FIRST NON-HEX MUST BE
	BZ ADDOUT              ;A LINEFEED OR
	XRI 007H               ;CTRN G BELL
	BZ PRMPT1
       XRI 02DH               ;DASH -
       BZ OLDDTA
       BR ERROR               ;ELSE ERROR

ADDOUT SEP CALL
       DW (OSTRNG)
       DB CR
       DB 0
       SEP CALL
       DW (OUT1)	;80CD

OLDDTA SEP DELAY
	DB 017H                ;WAIT TO FINISH READ
	LDN SRC
	PHI CHAR       ;STAY DN SAME LINE
	SEP CALL
	DW (TYPE2)  ;81AE           ;HEX OUTPUT
	SEP CALL	
	DW (OSTRNG) ;83F0          ;OUTPUT A HYPHEN
	DB '-'
	DB 0
	LDN SRC
	PLO ASL        ;COPY DATA FROM CELL INTO ASL

GETDTA SEP CALL
	DW READHX
	GLO ASL
	STR SRC        ;RESTORE THE DATA INTO THE CELL
       INC SRC                ;OPEN THE NEXT CELL
       BR DECODE              ;EXAMINE INPUT

OUT1 SEP DELAY;
       DB 017H
	GHI SRC
	PHI CHAR       ;ROUTINE TO OUTPUT A HEX PAIR
       SEP CALL;
       DW (TYPE2)             ;AND A SPACE
	GLO SRC
	PHI CHAR
       SEP CALL;
       DW (TYPE2)
       SEP CALL;
       DW (OSTRNG);
       DB SPACE
       DB 0
       SEP RETN

	DB	0FFH, 0FFH, 0FFH, 0FFH, 0FFH, 0FFH, 0FFH, 0FFH	;80DF-80E6

	DB	0FFH, 0FFH, 0FFH	;80E7-80E9
 ;
 ;DELAY IS 2(1 + AUX.l(3 + @LINX) & IS
 ;USED BY TYPE, READ, AND TIMALC . AUX.1
 ;IS ASSUMED TO HOLD A DELAY CONSTANT =
 ;((BIT TIHE OF TERMINAL)/
 ;(20 * INSTR TIHE OF COSMAC) - 1 . THIS
 ;CONSTANT CAN BE GENERATED AUTOMATICALLY
 ;BY THE T1MALC ROUTINE.
 ; REG USED: AUX, PC
 ;********************************************

DEXIT
	SEP DELAY
        SEP DELAY
        SEP DELAY
        SEP DELAY
 	SEP	PC		;80EE-D3                    


DELAY1
	GHI	AUX		;80EF-9E
	SHR			;80F0-F6
	PLO	AUX		;80F1-AE
DELAY2	DEC	AUX		;80F2-23
	LDA	PC		;80F3-43

DELAY3  SMI 1
	BNZ DELAY3
	GLO	RE
	BZ	DEXIT 
	DEC	PC		;80FB-23
	BR	DELAY2		;80FC-30 F2

;******************************* 
;CALCULATES BIT TIME AND ECHO FLAG. WAITS
;FOR LF(NO ECHO) OF CR (ECHO) TO BE TYPED
;IN. ALSO SETS UP POINTER TO THE DELAY
;ROUTINE. AUX.l ENDS UP HOLDING, IN THE
;HOST SIGNI FICANT 7 BITS , THE DELAY CONSTANT
;THE LEAST SIGNIFICANT BIT IS ZERO FOR ECHO
;OR ONE FOR NO ECHO .
; REG USED: AUX , CHAR
;***************************************************

	ORG UT62+00FEH

TIMALC GHI PC
	PHI DELAY       ;DELAY SUBROUTINE ADDRESS
       LDI LOW(DELAY1) ;80EFH
       PLO DELAY

	LDI	000H		;8103-F8 00
	PLO	AUX		;8105-AE
	PLO	CHAR		;8106-AF
R8107	B4	R8107		;8107-37 07
R8109	BN4	R8109		;8109-3F 09
	LDI	003H		;810B-F8 03
TC2	SMI	001H		;810D-FF 01
	BNZ	TC2		;810F-3A 0D
	GLO	CHAR		;8111-8F
	BNZ	ZERO		;8112-3A 17
	B4	INCR		;8114-37 19
	INC	CHAR		;8116-1F
ZERO	B4	DAUX		;8117-37 1E
INCR	INC	AUX		;8119-1E
	LDI	007H		;811A-F8 07
	BR	TC2		;811C-30 0D

DAUX	DEC	AUX		;811E-2E
	DEC	AUX		;811F-2E
	GLO	AUX		;8120-8E
	ORI	001H		;8121-F9 01
	PHI	AUX		;8123-BE
	SEP	DELAY		;8124-DC
	DB 	0CH		;8125-0C
	BN4	WAIT		;8126-3F 2C
	GHI	AUX		;8128-9E
	ANI	0FEH		;8129-FA FE
	PHI	AUX		;812B-BE
WAIT	SEP	DELAY		;812C-DC
	DB	'&'		;812D-26
	SEP	RETN		;812E-D5 RET

;     READS ONE BYTE INTO CHAR.L WHEN ENTERED
;READS ONE BYTE INTO CHAR. 1. WHEN ENTERED
;VIA READAH, ANY HEX INPUT IS ASSEMBLED
;INTO ASL AND OF - I, EL~E OF - 0 ON RETURN.
;NOTE-DON'T FOLLOW READ WITH IMMEDIATE TYPE.
;INSERT DELAY OR ENTER TYPE VIA TYPED.
;REG USED: CHAR, AUX & ASL
;****************************************************
       ORG UT62+012FH
 
CKDEC  ADI 7                      ;CHECK FOR ASCII DECIMAL
       BDF NFND                   ;OUT OF RANGE
       ADI 00AH                   ;SUBTRACT NET 30
       BDF FND	;FND

NFND   ADI 0                      ;SET DF = 0

REXIT  GHI CHAR                   ;PUT INPUT INTO D
       SEP RETN	;RETN             ; & RETURN

READAH LDI 0
	SKP                  ;SKIP TO READ1 HITH D=0

READ   GHI PC                     ; CONSTANT > 0
       PLO CHAR                   ;SAVE ENTRY POINT

READ2	LDI	080H		;8140-F8 80
	PHI	CHAR		;8142-BF
R8143	BN4	R8143		;8143-3F 43
R8145	B4	R8145		;8145-37 45
	SEP	DELAY		;8147-DC
	DB	02H		;8148-02
NOBIT	SEX	SP		;8149-E2
	SEX	SP		;814A-E2
BIT	GHI	AUX		;814B-9E
	SHR			;814C-F6
	BDF	READ3		;814D-33 57
	B4	OUTBIT		;814F-37 54
	SEQ			;8151-7B
	BR	NOECHO		;8152-30 59
OUTBIT	REQ			;8154-7A
	BR	NOECHO		;8155-30 59
READ3	NOP			;8157-C4
	NOP			;8158-C4
NOECHO	SEP	DELAY		;8159-DC
	DB	07H		;815A-07
	NOP			;815B-C4
	NOP			;815C-C4
	GHI	CHAR		;815D-9F
	SHR			;815E-F6
	PHI	CHAR		;815F-BF
	BDF	NEXT		;8160-33 69
	ORI	080H		;8162-F9 80
	BN4	NOBIT		;8164-3F 49
	PHI	CHAR		;8166-BF
	BR	BIT		;8167-30 4B
NEXT	REQ			;8169-7A
	BZ	READ2		;816A-32 40
	GLO	CHAR		;816C-8F
	BNZ	REXIT		;816D-3A 39
 	
CKHXE   GHI	CHAR		;816F-9F
	SMI	041H		;8170-FF 41 >A<
	BNF	CKDEC		;8172-3B 2F
	SMI	006H		;8174-FF 06
	BDF	NFND 		;8176-33 37
	ADI	010H		;8178-FC 10

FND	PLO	AUX		;817A-AE
	GHI	ASL		;817B-9D
	SHL			;817C-FE
	SHL			;817D-FE
	SHL			;817E-FE
	SHL			;817F-FE

	STR SP     ;SHL 4X
	GLO ASL
	SHR
	SHR
	SHR
	SHR
	OR
	PHI ASL
	GLO ASL
	SHL
	SHL
	SHL
	SHL
	STR	SP		;818D-52
	GLO	AUX		;818E-8E
	ANI	00FH		;818F-FA 0F
	OR			;8191-F1
	PLO	ASL		;8192-AD
	SMI	000H		;8193-FF 00
	BR	REXIT ;R8139		;8195-30 39

        DB    0FFH ;FILL

         ORG UT62+0198H
;TYPES ONE BYTE FROM CHAR.1 AS AN ASCII
;CHARACTER OR AS TWO HEX DIGITS LINE FEEDS
;ARE FOLLOWED BY SIX NULLS. USES REGISTERS
;AUX AND CHAR AND A STACK LOCATION . TYPED
;& TYPE5D ALLOW AN EARLIER READ TO COMPLETE.

;AUX.0 HOLDS OUTPUT CHARACTER (AT FIRST).
;THEN THE DELAY CONSTAHT BENEEN BITS.
;CHAR.0 HOLDS THE NUMBER OF BITS (11) IN
;ITS LOWER DIGIT AlfO THE FOLLOWING CODE IN
;ITS UPPER DIGIT:
;0 - BYTE OUTPUT
;1 FIRST HEX OUTPUT
;2 LAST NULL OUTPUT
;8 LF OUTPUT 


TYPED 
	SEP	DELAY		;8198-DC
	DB 	17H		;8199-17
	BR	TYPE		;819A-30 A4
TYPE5D
	SEP	DELAY		;819C-DC
	DB 	17H		;819D-17
	SKP	 		;819E-38 D5
TEXIT
	SEP	R5		;819F- D5 RET
TYPE5
	LDA	R5		;81A0-45
	SKP			;81A1-38 
TYPE6
	LDA	R6
	SKP			;81A3-38  

TYPE	GHI CHAR		;81A4-9F
	PLO	AUX		;81A5-AE
	XRI	00AH		;81A6-FB 0A
	BNZ	TY2		;81A8-3A BF
	LDI	08BH		;81AA-F8 8B
	BR	TY3		;81AC-30 C1

TYPE2
	GHI	CHAR
	SHR
	SHR
	SHR
	SHR
	 ADI 0F6H                ;CONVERT TO HEX
	 BNF TY4                 ;IF A OR >, ADD 37
	 ADI 7
R81B9
TY4     SMI 0C6H	;81B9-FF C6
	PLO	AUX		;81BB-AE
	LDI	01BH		;81BC-F8 1B
	LSKP			;81BE-C8

TY2     LDI 0BH                   ;NO OF BITS

TY3     PLO CHAR
	GLO	ASL
	STXD
BEGIN   SEQ
	GLO	AUX
	PLO	ASL
PREBIT  SEP	DELAY
	DB	07H
	DEC	CHAR
	SD
	GLO	ASL
	SHRC
	PLO	ASL
	BDF	OUT1A		;81CE 33 D3
	SEQ
	BR	OUT1B
OUT1A   REQ
	REQ
OUT1B   GLO	CHAR
	ANI	0FH
	NOP
	NOP
	NOP
	NOP
	BNZ	PREBIT

NXCHAR   GLO CHAR
	ADI 0FBH
	PLO CHAR
	 BNF FIXRD              ;SEP R5 IF NO MORE
	 SMI 01BH                ;TEST FOR ALTERNATIVES
	 BZ FIXRD               ;TYPED LAST NULL
	 BNF HEX1                ;TYPED FIRST HEX
	 LDI 0                   ;TYPED LF OR NULL
	 BR HEX3
	
HEX1     GHI CHAR
	ANI 00FH        ;GET 2ND HEX DIGIT
	 ADI 0F6H                 ;CONVERT TO HEX
	 LSNF                 ;IF A OR MORE.
	 ADI 7                    ;ADD NET 37

HEX2     SMI 0C6H                ;ELSE ADD NET 30

  
HEX3     PLO AUX                   ;AND SAVE
         BR BEGIN		;TYPE ROUTINE

FIXRD	INC	SP
	LDN	SP
	PLO	ASL
	BR	TEXIT  ;819FH

;*************************************************************************
;                  INPUT OPTION
;ALLOWS ENTRY OF EITHER STARTING AND ENDING
;ADDRESSES OR BYTE COUNT . EXITS WITH THE
;STARTING ADDRESS IN REG SRC AND THE BYTE
;COUNT IN REG CNT. RETURNS WITH DF-l
;IF SYNTAX ERROR EXISTS
;REG USED: ASL, SRC, CHAR, CNT
;*****************************************************                        

OPTION 	SEP CALL;
	DW (READHX)             ;GET THE STARTING ADDRESS
	GHI ASL
	PHI SRC         ;AND SAVE IT
	GLO ASL
	PLO SRC
	LDI 0
	PLO ASL
	PHI ASL   ;CLEAR THE INPUT REG
	GHI CHAR
	XRI SPACE      ;FIRST NONSMI HEX MUST
	BZ CNTIN                ;BE A SPACE OR
	XRI 00DH                ;A HYPHEN
	BNZ PRMPT2              ;ELSE SYNTAX ERROR
	SEP CALL;
	DW (READHX)             ;EXPECT ENDING ADDRESS

BYTCNT  GLO SRC
	STR SP          ;CALCULATE THE BYTE COUNT
	GLO ASL
	SM
	PLO CNT
	GHI SRC
	STR SP
	GHI ASL
	SMB
	PHI CNT
	BDF EXITOK              ;CHECK FOR SRC < ASL

INVERT  GLO ASL
	STR SP          ;ELSE EXCHANGE THE CONTENTS OF
	GLO SRC
	PLO ASL         ;SRC AND ASL
	LDN SP
	PLO SRC
	GHI ASL
	STR SP
	GHI SRC
	PHI ASL
	LDN SP
	PHI SRC
	BR BYTCNT               ;RECALCULATE

CNTIN   SEP CALL;
	DW (READHX)             ;INPUT THE BYTE COUNT
	GLO ASL
	SMI 1
	PLO CNT
	GHI ASL
	SMBI 0
	PHI CNT
	BDF EXITOK
	INC CNT
EXITOK  SEP RETN                  ;RETURN WHEN DONE

;*************************************************************************
;                    FILL ROUTINE
;        LOADS MEMORY BEGINNING AT ADDRESS CONTAINED 
;IN SRC WITH DATA CONTAINED IN ASL.0 FOF
;THE NUMBER OF BYTES SPECIFIED BY CNT.
;USER CALLABLE @ USRFlL .
; REG USED; ASL, SRC, CNT ,CHAR
;                      
FILL     SEP CALL;
	 DW (READAD)                  ;GET THE ADDRESSES
	 SEP CALL		      ;
	 DW (USRFIL)                  ;CALL THE MOVE

PRMPT2 LBR RENTER
	
NXTCEL   INC SRC                      ;POINT TO NEXT CELL
	DEC CNT			      ;REDUCE BYTE COUNT
				      ;LOAD THE DATA;USER ENTRY PT.
USRFIL   GLO ASL
	STR SRC              ;LOOP UNTIL COUNT = 0
	 GLO CNT
	 BNZ NXTCEL
	 GHI CNT
	 BNZ NXTCEL
	 SEP RETN                       ;EXIT THE CALL

;********************
; 'C' COMMAND
; ROUTINE TO READ FROH TAPE TO SCREEN
;AFTER EACH 20 RETURNS, ROUTINE OUTPUTS "*.**" AND
;WAITS FOR 8REAK KEY TO 8E HIT TO CONTINUE.
;HALTS ON BREAK, EXITS ON Q OR CONTINUE ON OTHER 

TPSCRN
	SEP	CALL
	DW	FINDTP		;8774H PRINT "TAPE #"
	GHI	DELAY
	STXD
	GLO	DELAY
	STXD
        SEP	CALL
	DW	OSTRNG
	DB	0AH
	DB	00H
	INC	SP
	LDA	SP
	PLO	DELAY
	LDN	SP
	PHI	DELAY
	LDI	LINES
	PLO	CNT
MORLST	SEP	CALL
	DW	READT

CHKOUT	GHI	DELAY
	STXD
	GLO	DELAY
	STXD
	GHI	CHAR
	XRI	EOF
	BZ	PRMPT2
	XRI	1EH
	BNZ	SHOWIT		;82A4H
	SEP	CALL
	DW	OSTRNG
	DB	0DH, 0AH, 00H
	DEC	CNT
	GLO	CNT
	BNZ	NXTLNE
	SEP	CALL
	DW	OSTRNG
	DB	'****' 
	DB	00H                     
R828A	SEP	CALL
	DW	READ	;813EH
	SEP	CALL
	DW	OSTRNG
	DB	0DH, 0AH, 00H
R8293	LDI	LINES	 
	PLO	CNT

NXTLNE	INC	SP
	LDXA
	PLO	DELAY
	LDN	SP
	PHI	DELAY
	SEP	CALL
	DW	READT
	XRI	LF
	BZ	MORLST
	BR	CHKOUT


SHOWIT  LDI LOW DELAY1  ;0EFH
	PLO DELAY
	LDI HIGH DELAY1   ;080H
	PHI DELAY
	SEP CALL
	DW	TYPE	;81A4H
	B4	MRLST1
	SEP	CALL
	DW	READ	;813EH
	XRI	'Q'
	BZ	PRMPT2

MRLST1	INC	SP
	LDXA
	PLO	DELAY
	LDN	SP
	PHI	DELAY
	BR	MORLST

;*************************************************************************
;                             OUTPUT
;     FORMATS AND OUTPUTS MEMORY DATA BEGINNING
; AT THE ADDRESS IN REG SRC FOR THE NUMBER
;OF BYTES SPECIFIED IN REG CNT
;  REG USED: SRC, CNT, CHAR
;****************************************************************              
DISPLY SEP CALL;
       DW (OPTION)               ;GET STARTING ADDRESS
       XRI CR                    ;TERMINATE WITH CR
       LBNZ ERROR

OUTPUT SEP CALL;
       DW (OSTRNG);
       DB LF
       DB 0                      ;START ON A NEW LINE
       SEP CALL;
       DW (OUT1)                 ;OUTPUT THE ADDRESS OF THE
       	  			 ;CURRENTLY OPENED CELL

SPCOUT SEP CALL
       DW (OSTRNG)
       DB SPACE
       DB 0


DATOUT LDA SRC
       PHI CHAR          ;RETRIEVE THE CELL DATA
       SEP CALL;
       DW (TYPE2)                ;AND OUTPUT IT
       GLO CNT                   ;DETERMINE IF THE
       BNZ NOTDON		 ;REGUESTED NO. OF BYTES              
R82DA  GHI CNT			 ;HAVE BEEN SENT
       BZ PRMPT2		 ;GET A NEW COMMAND


NOTDON DEC CNT			 ;DEC THE BYTE COUNT
       GLO SRC
       ANI LNECNT
       BNZ SAMELN		 ;END OF CURRENT LINE?
       SEP CALL;
       DW (OSTRNG)
       DB ';'
       DB CR
       DB 0
       BR OUTPUT
		
SAMELN SHR
       BDF DATOUT                ; WITHIN PAIR
       BR SPCOUT		 ;ELSE BETWEEN PAIRS
; READ 1 CHARACTER FROM TAPE AND DF=1 IF HEX
RDHEX	 SEP	CALL
	 DW	READT
	 SEP	CALL
	 DW	CKHXE
	 SEP	RETN
;UT62 MOVE COMMAND
;CALLS USRMOV AND REQUESTS SRC & DEST ADDRS
MOVE
	SEP  CALL              ;82F7: D4          
	DW   READAD           ;82F8: 83 03 
	 SEP	CALL
	 DW	USRMOV
	 LBDF ERROR
 	 LBR	PRMPT2


;*************************************************************************
;     SUBROUTINE TO GET THE ADDRESSES FOR OTHER ROUTINES
;*************************************************************************

READAD SEP CALL
       DW (OPTION)                  ;DETERMINE THE MODE
       XRI SPACE                    ;MUST BE A SPACE
       BNZ ERR1                     ;ELSE ERROR
	PLO ASL
	PHI ASL              ;CLEAR INPUT REGISTER
       SEP CALL;
       DW  READHX                  ;INPUT THE CONSTANT
       XRI CR                       ;'CR' TERMINATES
       BNZ ERR1                     ;ELSE ERROR
       SEP RETN
;*************************************************************************
;     FILLS ASL UNTIL A CARRIAGE RETURN IS ENTERED
;*************************************************************************

READCR SEP CALL 	
       DW  READAH  
       XRI CR
       BNZ READCR
       SEP RETN

;***
; READ A BIT OF DATA FROM TAPE
; ENTERED WITH SET P TO BITRDR

RBYE	 SEP	PC
RDBIT	 LDI	16H ;0/1 BIT DISTINGUISH TIMER
; FOR 2MHZ CRYSTAL
; THE TIMING LOOP USED GIVESW 22 x 24US DELAY (528)
; TO TEST FOR A 1/0 TRANSITION DATA FROM TAPE
; TYPICALLY HAS A 350/750US DIFFERENCE 0/1

R831F 	 BN2	R831F
	 SEP	BITRDR
	 BN2	RDBIT
HERE	 BN2	SLIDE1
HERE1	 SMI	01H
	 BNZ	HERE
HERE2	 INC	PARITY
	 GHI	PC
SLIDE2	 SHL
SLIDE3	 B2	SLIDE3
	 SEP	BITRDR
	 B2	SLIDE3
	 BR	RBYE

SLIDE1	 SMI	01H
	 BN2	SLIDE2
	 BNZ	HERE1
	 BR	HERE2

R833C	 DB 0FFH,0FFH,0FFH,0FFH,0FFH,0FFH,0FFH,0FFH 	; AND SO ON

	 DB 0FFH,0FFH,0FFH,0FFH,0FFH,0FFH,0FFH,0FFH  

	 DB 0FFH,0FFH,0FFH,0FFH,0FFH,0FFH,0FFH,0FFH	 	 

	 DB 0FFH,0FFH,0FFH,0FFH,0FFH,0FFH,0FFH,0FFH 	 

	 DB 0FFH

	 ORG	835DH

	 
PRMPT5  LBR RENTER
ERR1	LBR ERROR

;*************************************************************************
;DESC: STANDARD SEP CALL;,A( AND RETURN
;REG USED: SP,PC,SEP CALL;,A(,RETURN,LINK & STACK
;*************************************************************************

	ORG UT62+0363H

;                    STANDARD CALL

EXITC  SEP PC                       ;GO TO IT
CALLR  SEX SP 			    ;SET R(X)
	GHI LINK
	STXD                ;SAVE THE CURRENT LINK ON
	GLO LINK
	STXD                ;THE STACK
	GHI PC
	PHI LINK
	GLO PC
	PLO LINK
	LDA LINK
	PHI PC              ;PICK UP THE SUBROUTINE
	LDA LINK
	PLO PC              ;ADDRESS
	BR EXITC            ;STANDARD RETURN

EXITR  SEP PC                       ;RETURN TO MAIN PGM

RETR   GHI LINK
	PHI PC
	GLO LINK
	PLO PC
	SEX SP
	INC SP                ;SET THE STACK POINTER
	LDXA
	PLO LINK                ;RESTORE THE CONTENTS OF
	LDX
	PHI LINK                 ;LINK
	GHI CHAR                     ;PUT THE CONTENTS OF CHAR.1 INTO D
                                    ;BEFORE RETURNING
	BR EXITR

;*************************************************************************
;                REGISTER INITIALIZATION ROUTINE
; INITIALIZES REGISTER C TO THE DELAY ROUTINE, R2 AS A STACK
; POINTGER TO LOCATION 8CFFH, REG 4 TO CALL, REG 5 AS RETURN
; AND REG 3 AS PORGAM COUNTER
; ENTER 2 REG 3 MUST BE PRESET
; REG USED: PC, DELAY, CALL, RETURN, SP


INIT   LDI LOW(START)
	PLO PC
	LDI HIGH(START)
	PHI PC
       BR ENTER2

ENTER1 LDI LOW(PGMSRT)
	PLO PC
	LDI HIGH(PGMSRT)
	PHI PC


ENTER2 LDI LOW(DELAY1)
	PLO DELAY       ;DELAY ROUTINE
	LDI HIGH(DELAY1)
	PHI DELAY
	LDI HIGH(CALLR)
	PHI CALL
	PHI RETN
	LDI LOW(CALLR)
	PLO CALL
	LDI LOW(RETR)
	PLO RETN
	LDI LOW(TOPSTK)
	PLO SP
	LDI HIGH(TOPSTK)
	PHI SP
	SEX SP
	SEP PC

;*************************************************************************
;                 HEX BYTE INSERT ROUTINE
;INSERTS HEX PAIRS INTO MEMORY STARTING AT A SPECIFIED
;ADDRESS. AFTER A ";" ALL IS IGNORED UNTIL A RETURN
;THEN A NEW ADDRESS IS EXPECTED. ANY NON-HEX DATA IS
;IGNORED BETWEEN HEX PAIRS BUT NOTHING IS PERMITTED
;BETWEEN MEMBERS OF THE PAIR . ROUTINE IS TERMINATED
;WITH A RETURN, EXCEPT AFTER A ";".
;REG USED: ASL, SRC, CHAR
;***********************************************************
 
INSERT
	LDI	0
	PHI	ASL
	PLO	ASL                 ;CLEAR INPUT REGISTER
FSTHEX	SEP	CALL
	DW	READHX
	XRI	' '
	BNZ 	ERR1              ;IT MUST BE A SPACE
	GHI	ASL
	PHI	SRC
	GLO	ASL
	PLO	SRC                   ;INPUTS UERE THE STARTING ADDRESS
	
NXTCHR  SEP	CALL
	DW	READAH
	BNF NTDATA                     ;IF NEXT INPUT IS HEX
	SEP	CALL
	DW	READAH                    ;GET A SECOND
	BNF ERR1                       ;WHICH MUST ALSO BE HEX
	GLO	ASL
	STR	SRC
	INC SRC            ;AND STORE HEX PAIR INTO MEMORY
	BR NXTCHR                      ;LOOK FOR MORE


NTDATA  XRI	CR
	BZ PRMPT5             ;IF INPUT HAS CR5 LEAVE
	GHI	CHAR
	XRI	SEMCOL
	BNZ	COMCHK                  ;IF INPUT HAS ';',
	SEP	CALL
	DW	CKCRLF                  ;IGNORE EVERYTHING UNTIL CR
	BR	INSERT

COMCHK	GHI	RF                
	XRI	COMMA	 	 
	BNZ	NXTCHR                     ;IF INPUT HAS ',',
	SEP	CALL
	DW	CKCRLF                   ;IGNORE EVERYTHING UNTIL CR
	BR	NXTCHR

CKCRLF	SEP	CALL
	DW	READ
	XRI	CR
	BNZ	ERR1
	SEP	CALL
	DW	READ
	XRI	LF
	BNZ	ERR1
	SEP	R5
 
	DB 0FFH, 0FFH, 0FFH, 0FFH
 
;*************************************************************************
;                      UTILITY ENTRY TABLE
;*************************************************************************

        ORG UT62+03F0H

OSTRNG  LBR MSGE
INIT1   LBR ENTER1
INIT2   LBR ENTER2
GOUT62  LBR RENTER
CKHEX   LBR CKHXE

;*************************************************************************
;                        SELECT GROUP 1
;*************************************************************************


TPOFF   SEX PC
	OUT BDSEL
	DB TAPEIO	;SELECT TAPE BD
	OUT TPESEL
	DB TPEOFF	;OUTPUT OFF BITS
	OUT BDSEL
	DB TRMINL	;SELECT RCA GRP
	SEP RETN	;RETURN TO CALLER


; REGISTER SAVE ROUTINES
;SAVES THE CONTENTS OF REGISTERS TMPRGl (7),
;TMPRC2 (8), TMPRG3 (9) AND CNT (A) ON THE STACK.
;USES 8 BYTES OF STACK, AND LEAVES STACK POINTER
;AT A FREE BYTE  
;*************** *** ******
REGSAV
	GLO  CNT              ;8407: 8A          
	STXD                 ;8408: 73          
	GHI  CNT              ;8409: 9A          
	STXD                 ;840A: 73          
	GLO  TMPRG3              ;840B: 89          
	STXD                 ;840C: 73          
	GHI  TMPRG3              ;840D: 99          
	STXD                 ;840E: 73          
	GLO  TMPRG2              ;840F: 88          
	STXD                 ;8410: 73          
	GHI  TMPRG2              ;8411: 98          
	STXD                 ;8412: 73          
	GLO  TMPRG1              ;8413: 87          
	STXD                 ;8414: 73          
	GHI  TMPRG1              ;8415: 97          
	STXD                 ;8416: 73          
	GHI  CHAR              ;8417: 9F          
	SEP  R5              ;8418: D5
          
MSGE	;OSTRNG

	LDI	LOW(DELAY1)   ;0EFH
	PLO	DELAY
	LDI	HIGH(DELAY1)	;80H
	PHI	DELAY
MSGE1
	LDA	LINK
	PHI	CHAR
	BZ	EXITM
	SEP	CALL
	DW	TYPED			;8198H
	BR	MSGE1
EXITM
	SEP	R5

;*************************************************************************
;                  MOVE ROUTINE
;COPIES A BLOCK OF MEMORY FORM ONE CONTINUOUS AREA
;TO ANOTHER CONTINUOUS AREA IN MEMORY. THERE IS NO
;RESTRICTION AS TO THE DIRECTION OF THE MOVE AND THE
;AREAS HAY OVERLAP .
;REG USED: SRC. DEST, CHAR. & CNT
 
USRMOV   SEX SP
	GLO SRC
	STR SP               ;TEST THE RELATIVE POSITION
	GLO DEST
	SM                  ;OF SOURCE & DESTINATION
	 BNZ DIRECT                   ;NOT EQUAL!       
	GHI SRC
	STR SP               ;RETURN IF THEY ARE EQUAL
	GHI DEST
	SMB
	 BZ USRBYE                    ;EXIT TO CALLER

DIRECT   GLO SRC
	STR SP               ;ELSE TEST FOR UP OR DOWN
	GLO DEST
	SM                  ;DIRECTION OF THE MOVE
	GHI SRC
	STR SP
	GHI DEST
	SMB
	 BDF MOVUP
MOVDN    LDN SRC
	STR DEST             ;DO THE MOVE DOWN AND
	 GLO CNT                      ;AND CHECK IF DONE
	 BNZ MOVDN1
	 GHI CNT
	 BZ USRBYE                    ;EXIT TO CALLER
MOVDN1   INC SRC
	INC DEST             ;ADJUST THE POINTERS
	 DEC CNT                      ;REDUCE THE BYTE COUNT             
	 BR MOVDN                     ;FINISHED

MOVUP    GLO CNT
	STR SP               ;SET THE POINTERS TO THE
	GLO SRC
	ADD
	PLO SRC          ;TOP OF MOVE AREAS
	GHI CNT
	STR SP
	GHI SRC
	ADC
	PHI SRC
	GLO CNT
	STR SP
	GLO DEST
	ADD
	PLO DEST
	GHI CNT
	STR SP
	GHI DEST
	ADC
	PHI DEST
	 BNF UP

ERRGO    BR USRBYE+1                  ;EXIT DF=1 IF OVERFLON

UP       LDN SRC
	STR DEST             ;DO THE MOVE UP
	 GLO CNT                      ;AND CHECK IF DONE
	 BNZ UP1
	 GHI CNT
	 BZ USRBYE                    ;EXIT TO CALLER

UP1      DEC SRC
	DEC DEST
	DEC CNT     ;ADJUST THE POINTERS
	 BR UP

USRBYE   SHR                          ;SET DF=0 IF A<FFFF
	 SEP RETN                       ;EXIT TO CALLER 

; 'R' COMMAND, maybe tape read? 
;   TAPE LOADER ROUTINE
; LOADS USER KEHOII.Y FROM TAPE, ASSUMES TAPE HAS M OR I TO 
; INDICATE START OF DATA TO BE STORED. "P" FOLLOWED BY AN ADDRESS
; STARTS PROGRAM EXECUTION WITH P=0 AND X=0
  
TPLOAD
	 SEP CALL
	 DW	FINDTP
	 GHI	DELAY
	 STXD
	 GLO	DELAY
	 STXD
	 SEP	CALL
	 DW	OSTRNG
	 DB	0AH
	 DB	'LOADING',00H



		INC  SP              ;8487: 12          
		LDXA                 ;8488: 72          
		PLO  DELAY              ;8489: AC          
		LDN  SP              ;848A: 02          
		PHI  DELAY              ;848B: BC          
READZ
		SEP  CALL             ;READ 1 ASCII DIGIT      
		DW   READT                          
		XRI  'P'            ;CHECK FOR PROGRAM  EXECUTE
		BZ   ADLP               
		XRI  1DH            ;SET MEMORY?
		LSZ                        
		XRI  04H            ;INSERT MEMORY?
		BZ   READX1               
		XRI  5AH             ;CHECK FOR EOF (DC3)    
		BZ   PRMPT4             
		BR   READZ          

                ; READ THE HEX ADDRESS      
READX1
		SEP  CALL              ;84A0: D4          
		DW   RDHEX           ;84A1: 82 F0
		BDF  READX2        ;84A3: 33 B2       
		XRI  2EH             ;84A5: FB 2E       
		BNZ  READX1           ;84A7: 3A A0       
READXA
		SEP  CALL              ;84A9: D4          
		DW   READT          ;84AA: 85 42
		XRI  CR            ;84AC: FB 0D       
		BNZ  READXA         ;84AE: 3A A9       
		BR   READX1          ;84B0: 30 A0       
READX2
		SEP  CALL            ;READ NEXT ASCII CHAR         
		DW   RDHEX            
		BDF  READX2          ;IF HEX KEEP READING    
		XRI  SPACE           ;OTHERWISE MUST BE A SPACE      
		BNZ  ERRORX                  
		GHI  ASL                      
		PHI  ADRPTR              
		GLO  ASL                    
        PLO  ADRPTR
READX3
		SEP  CALL            ;READ NEXT ASCII CHAR     
		DW   RDHEX           ;IF HEX KEEP READING
		BDF  READXB           ;84C2: 33 CE       
		XRI  CR             ;84C4: FB 0D       
		BZ   READZ           ;84C6: 32 8C       
		XRI  36H             ;84C8: FB 36       
		BZ   READXA          ;84CA: 32 A9       
		BR   READX3         ;84CC: 30 BF       
READXB
		SEP  CALL              ;READ 2ND ASCII CHAR         
		DW   RDHEX            
		BNF  ERRORX         ;BETTER BE A HEX DIGIT    
		GLO  ASL                       
		STR  ADRPTR          ;STORE AT THE SPECIFIED ADDR          
		SEX  ADRPTR           ;MAKE SURE THERE IS RAM THERE          
		XOR                            
		BZ   WRTOK               
		SEP  CALL                     
		DW   NOTRAM        
WRTOK
		INC  ADRPTR              ;84DC: 18          
		BR   READX3          ;84DD: 30 BF 

                ; ROUTINE TO START A PROGRAM RUNNING      
ADLP
		SEP  CALL              ;84DF: D4          
		DW   RDHEX           ;GET THE START ADDR
		BDF  ADLP                  
		LBR  RUN1           ;  

                ; ERROR MESSAGE
ERRORX
		SEP  CALL              ;84E7: D4          
		DW  OSTRNG          ;84E8: 83 F0
		DB   0DH             ;84EA: 0D
		DB   0AH             ;84EB: 0A
		DB   'FORMAT ERROR'  


		DB   00H             ;84F8: 00

PRMPT4
		LBR  RENTER           ;84F9: C0 87 F0  

		DB   0FFH            ;84FC: FF
		DB   0FFH            ;84FD: FF
		DB   0FFH            ;84FE: FF
		DB   0FFH            ;84FF: FF

;   USER ENTRY POINTS FOR TAPE ROUTINES

                ORG 8500H

S8500
		BR   TWRITE           ;TO TAPE WRITE 
S8502
		BR   READT         ;TO TAPE READ      
REWIND
		SEP  CALL              ;8504: D4          
		DW   OSTRNG           ;8505: 83 F0
		DB   0DH             ;8507: 0D
		DB   0AH             ;8508: 0A
		DB   'REWIND,THEN HIT ANY KEY'  





		DB   00H             ;8520: 00
S8521
		SEX  PC              ;8521: E3          
		OUT  BDSEL              ;8522: 61          
		DB   TAPEIO          ;8523: 02
		OUT  TPESEL              ;8524: 64          
		DB   BTHDRV          ;8525: C0
		OUT  BDSEL               ;      
		DB   TRMINL          
		SEP  CALL                   
		DW   READ	      ;WAIT FOR KEY              
		LBR  TPOFF            
 
TPWIND
		SEP  CALL              ;852E: D4          
		DW   REWIND         ;FOR THE UTILITY CALL J
S8531
		LBR  RENTER           ;8531: C0 87 F0    
S8534
;*************************************************************************
;       STARTS A USER PROGRAM WITH SPECIFIED ADDRESS
;       IN REGISTER 0 AND X=O. THE SCREEN IS CLEARED
;       AND THE CURSOR HOMED
;REG USED: CHAR, ASL, R0
;*************************************************************************

RUN     SEP CALL;
	DW (READHX)             ;LOOK FOR STARTING ADDRESS
	XRI CR                  ;FIRST NON-HEX MUST BE A
	LBNZ ERROR              ;CR, ELSE SYNTAX ERROR

RUN1   GHI ASL
	PHI R0           ;GET THE ADDRESS
	GLO ASL
	PLO R0
       SEX R0
       SEP R0                   ;AND GO!

;
;                  READ TAPE ROUTINE
;INITIALLY READS ONE BLOCK OF 384 BYTES INTO A BUFFER FROM TAPE
;THEN EACH CALL TO IT PUTS ONE BYTE INTO CHAR.l AND DECREKENTS
;DOWN THE BUFFER UNTILL IT IS EMPTY THEN GETS NEXT BUFFER LOAD
;USES A 3 BYTE BUFFER WHICH WEN NOT IN USE IS STORED ON STACK
;     LOWEST ADDRESS IS BLOCK COUNT
;     NEXT IS DRIVE 1 IN TOP2 BITS AND MSB OF BYTE COUNT
;     HIGHEST IS REST OF BYTE COUNT
;PTR (REGISTER C) NORMALLY POINTS AT THE HIGHEST BYTE
;NOTE: REGISTER C MUST CONTAIN THE IOCB ADDRESS AS SET UP
;    IN THE ROt1TINE FINDTP.
;******************************************
      
READT	
		GHI  PC              ;8542: 93          
		STXD                 ;8543: 73          
		LDN  PTR             ;8544: 0C          
		BNZ  EXRD           ;8545: 3A 7B       
		DEC  PTR              ;8547: 2C          
		LDN  PTR              ;8548: 0C          
		SHR                  ;8549: F6          
		BDF  EXRD1           ;854A: 33 79       
		LDN  PTR              ;854C: 0C          
		ORI  01H             ;854D: F9 01       
		STR  PTR              ;854F: 5C          
		INC  PTR              ;8550: 1C          
		LDI  80H             ;8551: F8 80       
		STR  PTR              ;8553: 5C          
		SEP  CALL              ;8554: D4          
		DW   RDBLOK          ;8555: 86 76
S8557
		BNF  TAG1           ;8557: 3B 7A       
		INC  SP              ;8559: 12          
		LDI  00H             ;855A: F8 00       
		STXD                 ;855C: 73          
		GHI  PTR              ;855D: 9C          
		STXD                 ;855E: 73          
		GLO  PTR              ;855F: 8C          
		STXD                 ;8560: 73          
		SEP  CALL              ;8561: D4          
		DW   OSTRNG          ;8562: 83 F0
		DB   0DH             ;8564: 0D
		DB   0AH             ;8565: 0A
		DB   'PARITY BAD'    ;8566: 50


		DB   0DH             ;8570: 0D
		DB   0AH             ;8571: 0A
		DB   00H             ;8572: 00

		INC  SP              ;8573: 12          
		LDA  SP              ;8574: 42          
		PLO  PTR              ;8575: AC          
		LDN  SP              ;8576: 02          
		PHI  PTR              ;8577: BC          
		SKP                  ;8578: 38          
EXRD1
		INC  PTR              ;8579: 1C          
TAG1
		LDN  PTR              ;857A: 0C          
EXRD
		SMI  01H             ;857B: FF 01       
		STR  PTR              ;857D: 5C          
		PLO  CHAR              ;857E: AF          
		DEC  PTR              ;857F: 2C          
		LDN  PTR              ;8580: 0C          
		SMBI 00H             ;8581: 7F 00       
		STR  PTR              ;8583: 5C          
		ANI  01H             ;8584: FA 01       
		ADI  (HIGH RDBUFF)-1  ;8586: FC 8D       
		PHI  CHAR              ;8588: BF          
		INC  PTR              ;8589: 1C          
		LDN  CHAR              ;858A: 0F          
		PHI  CHAR              ;858B: BF          
		INC  SP              ;858C: 12          
		LDN  SP              ;858D: 02          
		SHL                  ;858E: FE          
		SEP  RETN              ;858F: D5 

;TAPE WRITE ROUTINE
; PUTS BYTE FROM CHAR.1 INTO BLOCK BUFFER
; WHEN BUFFER FULL, WRITES IT TO SELECTED TAPE
; NOTE: REG C MUST CONTAIN THE IOCB ADDRESS

TWRITE
		GHI  CHAR              ;8590: 9F          
		STR  SP              ;8591: 52          
		LDN  PTR              ;8592: 0C          
		ADI  LOW WRTBUF             ;8593: FC 80       
		PLO  CHAR              ;8595: AF          
		DEC  PTR              ;8596: 2C          
		LDN  PTR              ;8597: 0C          
		ANI  01H             ;8598: FA 01       
		ADCI HIGH WRTBUF    ;859A: 7C 8E       
		PHI  CHAR              ;859C: BF          
		LDN  SP              ;859D: 02          
		STR  CHAR              ;859E: 5F          
		INC  CHAR              ;859F: 1F          
		INC  PTR              ;85A0: 1C          
		LDN  PTR              ;85A1: 0C          
		ADI  01H             ;85A2: FC 01       
		STR  PTR              ;85A4: 5C          
		DEC  PTR              ;85A5: 2C          
		LDN  PTR              ;85A6: 0C          
		ADCI 00H             ;85A7: 7C 00       
		STR  PTR              ;85A9: 5C          
		INC  PTR              ;85AA: 1C          
		GHI  CHAR              ;85AB: 9F          
		XRI  90H             ;85AC: FB 90       
		BNZ  EXWT          ;85AE: 3A C2       
		SEP  CALL              ;85B0: D4          
		DW   WRBLOK           ;85B1: 85 FE
 
		DEC  PTR              ;85B3: 2C          
		DEC  PTR              ;85B4: 2C          
		LDN  PTR              ;85B5: 0C          
		ADI  01H             ;85B6: FC 01       
		STR  PTR              ;85B8: 5C          
		INC  PTR              ;85B9: 1C          
		LDN  PTR              ;85BA: 0C          
		ANI  0FEH            ;85BB: FA FE       
		STR  PTR              ;85BD: 5C          
		INC  PTR              ;85BE: 1C          
		LDI  00H             ;85BF: F8 00       
		STR  PTR              ;85C1: 5C          
EXWT
		SMI  00H             ;85C2: FF 00       
		SEP  RETN              ;85C4: D5 

; MESSAGE FOR NO RAM AT A LOCATION FOR STORE OF A READ

NOTRAM
		GHI  PTR              ;85C5: 9C          
		STXD                 ;85C6: 73          
		GLO  PTR              ;85C7: 8C          
		STXD                 ;85C8: 73          
		SEP  CALL              ;85C9: D4          
		DW   OSTRNG           ;85CA: 83 F0
		DB   0DH             ;85CC: 0D
		DB   0AH             ;85CD: 0A
		DB   'NO RAM @'      ;85CE: 4E

		DB   00H             ;85D6: 00
		GHI  ADRPTR              ;85D7: 98          
		PHI  CHAR              ;85D8: BF          
		SEP  CALL              ;85D9: D4          
		DW   TYPE2           ;85DA: 81 AE
		GLO  ADRPTR             ;85DC: 88          
		PHI  CHAR              ;85DD: BF          
		SEP  CALL              ;85DE: D4          
		DW   TYPE2           ;85DF: 81 AE
		INC  SP              ;85E1: 12          
		LDXA                 ;85E2: 72          
		PLO  PTR              ;85E3: AC          
		LDN  SP              ;85E4: 02          
		PHI  PTR              ;85E5: BC          
		SEP  RETN              ;85E6: D5 

; ROM ENTRY FROM UTILITY

;******************************
; 'E' COMMAND, jump to ROM EDITOR 9000H
         
EDITOR
		LDI  AEDIT             ;85E7: F8 90       
		PHI  R0              ;85E9: B0          
		BR   GOADD           ;85EA: 30 F9 

;********************
; 'A' COMMAND, jump to ROM ASSEMBLER 9B00H 
    
ASSEMB
		LDI  AASM             ;85EC: F8 9B       
		PHI  R0              ;85EE: B0          
		BR   GOADD          ;85EF: 30 F9

;*******************
; 'B' COMMAND, jump to ROM B000H
       
BASIC
		LDI  ABAS           ;85F1: F8 B0       
		PHI  R0              ;85F3: B0          
		BR   GOADD           ;85F4: 30 F9       

;*******************
;'Z' COMMAND, jump to ROM E000H

PROMIT		LDI  PRMPGR            ;85F6: F8 E0       
		PHI  R0              ;85F8: B0 

;**************
; PROCESS E, A, B, Z commands with jump
         
GOADD
		LDI  00H             ;85F9: F8 00       
		PLO  R0              ;85FB: A0          
		SEX  R0              ;85FC: E0          
		SEP  R0              ;GOTO ROM

;****************
; WRITE BLOCK ROUTINE
; WRITES A BLOCK OF 384 BYTES TO TAPE  
  
WRBLOK
		SEP  CALL              ;85FE: D4          
		DW   REGSAV           ;85FF: 84 07               
		GLO  RB              ;8601: 8B          
		STR  SP              ;8602: 52          
		LDI  LOW WRTBUF      ;8603: F8 80       
		PLO  BUFPTR              ;8605: AA          
		LDI  HIGH WRTBUF     ;8606: F8 8E       
		PHI  BUFPTR   ;RA point to 8E80H  ;8608: BA          
		PLO  BLKFLG              ;8609: AB          
		LDI  LOW WRBIT             ;860A: F8 59       
		PLO  BITWRT              ;860C: A8          
		LDI  HIGH WRBIT             ;860D: F8 86       
		PHI  BITWRT   ;R8 point to 8659H   ;860F: B8          
		SEX  PC              ;8610: E3          
		OUT  BDSEL               ;8611: 61          
		DB   TAPEIO             ;8612: 02
		SEX  PTR              ;8613: EC          
		DEC  PTR              ;8614: 2C          
		OUT  TPESEL              ;8615: 64          
		DEC  PTR              ;8616: 2C          
		DEC  PTR              ;8617: 2C          
		LDN  PTR              ;8618: 0C          
		BNZ  LDR2           ;8619: 3A 1E       
		LDI  0C0H            ;861B: F8 C0       
		LSKP                 ;861D: C8          
LDR2
		LDI  10H             ;861E: F8 10       
		PHI  LDRCNT              ;8620: B9          
LEADER
		ADI  00H             ;8621: FC 00       
		SEP  BITWRT          ;jump 8659H now    ;8623: D8          
 
		DEC  LDRCNT              ;8624: 29          
		GHI  LDRCNT              ;8625: 99          
		BNZ  LEADER           ;8626: 3A 21  


     
SET1
		LDI  10H             ;8628: F8 10       
		PLO  PARITY              ;862A: A7          
		LDI  08H             ;862B: F8 08       
		PLO  BITCNT              ;862D: A9          
		GLO  BLKFLG              ;862E: 8B          
		BZ   NOTBLK           ;862F: 32 37       
		LDA  PTR              ;8631: 4C          
		PHI  BYTSTR              ;8632: B7          
		LDI  00H             ;8633: F8 00       
		PLO  BLKFLG              ;8635: AB          
		LSKP                 ;8636: C8          
NOTBLK
		LDA  BUFPTR              ;8637: 4A          
		PHI  BYTSTR             ;8638: B7          
		SMI  00H             ;8639: FF 00       
		SEP  BITWRT              ;863B: D8 
        
SET
		GHI  BYTSTR              ;863C: 97          
		SHR                  ;863D: F6          
		PHI  BYTSTR              ;863E: B7          
		SEP  BITWRT              ;863F: D8          
 
		DEC  BITCNT              ;8640: 29          
		GLO  BITCNT              ;8641: 89          
		BNZ  SET              ;8642: 3A 3C       
		INC  BYTSTR              ;8644: 17          
		GLO  BYTSTR              ;8645: 87          
		SHR                  ;8646: F6          
		SEP  BITWRT              ;8647: D8  
        
 
		GHI  BUFPTR              ;8648: 9A          
		XRI  90H             ;8649: FB 90       
		BNZ	SET1           ;864B: 3A 28       
		SEP  BITWRT              ;864D: D8   
       
 
		SEP  BITWRT              ;864E: D8          
		INC  PTR              ;864F: 1C          
		LDN  SP              ;8650: 02          
		PLO  RB              ;8651: AB          
RESTOR
		SEP  CALL              ;8652: D4          
		DW   REGSTR            ;8653: 87 D7                  
		LBR	TPOFF           ;8655: C0 83 FF 

;ROUTINE TO WRITE A BIT ONTO TAPE
;ENTERED WITH SEP TO BITWRTR

BYE
		SEP  PC              ;8658: D3                              
 
WRBIT	SEX  BITWRT              ;8659: E8          
		LDI  0FFH            ;865A: F8 FF       
		PLO  FLAG              ;865C: AF          
		LDI  0CH             ;865D: F8 0C       
		BNF  OUTAPE         ;865F: 3B 64       
		LDI  27H             ;8661: F8 27       
		INC  PARITY            ;8663: 17          
OUTAPE
		OUT  TPEWRT               ;8664: 65          
		DB   01H             ;8665: 01
		PHI  CHAR            ;8666: BF          
TEST1
		SMI  01H             ;8667: FF 01       
		BNZ  TEST1           ;8669: 3A 67       
		GLO  FLAG              ;866B: 8F          
		BZ   BYE           ;866C: 32 58       
		LDI  00H             ;866E: F8 00       
		PLO  FLAG              ;8670: AF          
		OUT  TPEWRT               ;8671: 65          
		DB   00H             ;8672: 00
		GHI  CHAR              ;8673: 9F          
		BR   TEST1           ;8674: 30 67 
;
; READ BLOCK ROUTINE
;  READS ONE BLOCK 384 BYTES, FROM TAPE INTO READ BUFFER
;  EXITS WITH DF AS PARITY FLAG, 0 IS GOOD

RDBLOK
		SEP  CALL              ;8676: D4          
		DW   REGSAV            ;8677: 84 07
 
		LDI  LOW RDBUFF            ;8679: F8 7F       
		PLO  BUFPTR              ;867B: AA          
		LDI  HIGH RDBUFF             ;867C: F8 8E       
		PHI  BUFPTR  ;RA is 8E7FH    ;867E: BA          
		PHI  FLAG             ;867F: BF          
		LDI  00H             ;8680: F8 00       
		PLO  FLAG  ;RF is 8E00H     ;8682: AF          
		LDI  LOW RDBIT             ;8683: F8 1D       
		PLO  R8              ;8685: A8          
		LDI  HIGH RDBIT             ;8686: F8 83       
		PHI  R8  ;R8 is 831DH, I/O on B2    ;8688: B8          
		SEX  PC              ;8689: E3          
		OUT  BDSEL               ;868A: 61          
		DB   TAPEIO             ;868B: 02
		DEC  PTR                ;868C: 2C          
		LDN  PTR              ;868D: 0C          
		SHL                  ;868E: FE          
		INC  PTR               ;868F: 1C          
		BDF  DR1           ;8690: 33 95       
		OUT  TPESEL               ;8692: 64          
		DB   DRIVE0             ;8693: 40
		LSKP                 ;8694: C8          
DR1
		OUT  TPESEL               ;8695: 64          
		DB   DRIVE1            ;8696: 80
LDR1
		LDI  08H             ;8697: F8 08       
		PHI  LDRCNT             ;8699: B9          
LEDER1
		SEP  BITRDR              ;869A: D8                          
		BDF  LDR1          ;869B: 33 97       
		DEC  LDRCNT              ;869D: 29          
		GHI  LDRCNT              ;869E: 99          
		BNZ  LEDER1           ;869F: 3A 9A       
STRT1
		SEP  BITRDR             ;86A1: D8  
		BNF  STRT1           ;86A2: 3B A1       
		LDI  09H             ;86A4: F8 09       
		PLO  BITCNT              ;86A6: A9          
		PLO  PARITY              ;86A7: A7          
SHIFT
		GHI  TMPRG1              ;86A8: 97          
		SHRC                 ;86A9: 76          
		PHI  TMPRG1              ;86AA: B7          
		DEC  BITCNT              ;86AB: 29          
		SEP  BITRDR             ;86AC: D8          
 
		GLO  BITCNT              ;86AD: 89          
		BNZ  SHIFT          ;86AE: 3A A8       
		GLO  PARITY              ;86B0: 87          
		SHR                  ;86B1: F6          
		LSDF                 ;86B2: CF          
		GHI  PC              ;86B3: 93          
		PLO  FLAG             ;86B4: AF          
		GHI  FLAG              ;86B5: 9F          
		BZ   BYTE           ;86B6: 32 C3       
		DEC  PTR              ;86B8: 2C          
		DEC  PTR              ;86B9: 2C          
		GHI  TMPRG1              ;86BA: 97          
		STR  PTR              ;86BB: 5C          
		INC  PTR              ;86BC: 1C          
		INC  PTR              ;86BD: 1C          
		LDI  00H             ;86BE: F8 00       
		PHI  FLAG              ;86C0: BF          
		BR   STRT1           ;86C1: 30 A1       
BYTE
		GHI  TMPRG1              ;86C3: 97          
		STR  BUFPTR              ;86C4: 5A          
		DEC  BUFPTR              ;86C5: 2A          
		GLO  BUFPTR              ;86C6: 8A          
		XRI  0FFH            ;86C7: FB FF       
		BNZ  STRT1           ;86C9: 3A A1       
		GHI  BUFPTR              ;86CB: 9A          
		XRI  8CH             ;86CC: FB 8C       
		BNZ  STRT1           ;86CE: 3A A1       
		GLO  FLAG             ;86D0: 8F          
		SHL                  ;86D1: FE          
		BR   RESTOR           ;86D2: 30 52 

;************************
; TAPE SAVE ROUTINE, DB 'W' command
; COPIES DATA FROM MEMORY TO TAPE  
TPWRIT
		SEP  CALL              ;86D4: D4          
		DW OSTRNG           ;86D5: 83 F0
		DB   0DH             ;86D7: 0D
		DB   0AH             ;86D8: 0A
		DB   'FROM '         ;86D9: 46

		DB   00H             ;86DE: 00
 
		LDI  00H             ;86DF: F8 00       
		PHI  SRTADD              ;86E1: B7          
		PLO  SRTADD              ;86E2: A7          
		PLO  TMPRG3              ;86E3: A9          
		PHI  ASL              ;86E4: BD          
		PLO  ASL              ;86E5: AD          
		SEP  CALL              ;86E6: D4          
		DW   READCR         ;86E7: 83 14
 
		GHI  ASL              ;86E9: 9D          
		PHI  SRTADD              ;86EA: B7          
		GLO  ASL             ;86EB: 8D          
		PLO  SRTADD              ;86EC: A7          
		LDI  00H             ;86ED: F8 00       
		PLO  ASL              ;86EF: AD          
		PHI  ASL              ;86F0: BD          
		SEP  CALL              ;86F1: D4          
		DW OSTRNG           ;86F2: 83 F0
		DB   0AH             ;86F4: 0A
		DB   'TO '           ;86F5: 54
		                     ;86F6: 4F
		                     ;86F7: 20
		DB   00H             ;86F8: 00
		SEP  CALL              ;86F9: D4          
		DW   READCR           ;86FA: 83 14
S86FC
		GHI  ASL              ;86FC: 9D          
		PHI  ENDADD              ;86FD: B8          
		GLO  ASL             ;86FE: 8D          
		PLO  ENDADD              ;86FF: A8          
		INC  ENDADD              ;8700: 18          
		SEP  CALL              ;8701: D4          
		DW   FINDTP           ;8702: 87 74
S8704
		SEP  CALL              ;8704: D4          
		DW   WRITEX          ;8705: 87 A6
		DB   'I'             ;8707: 49
		DB   0DH             ;8708: 0D
		DB   0AH             ;8709: 0A
		DB   00H             ;870A: 00
S870B
		SEP  CALL              ;870B: D4          
		DW   WRTH           ;870C: 87 52
CKEND
		GLO  ENDADD              ;870E: 88          
		STR  SP              ;870F: 52          
		GLO  SRTADD              ;8710: 87          
		XOR                  ;8711: F3          
		BNZ  WRTD2           ;8712: 3A 1A       
		GHI  ENDADD              ;8714: 98          
		STR  SP              ;8715: 52          
		GHI  SRTADD              ;8716: 97          
		XOR                  ;8717: F3          
		BZ   FINISH           ;8718: 32 30       
WRTD2
		GLO  SRTADD              ;871A: 87          
		ANI  1FH             ;871B: FA 1F       
		BNZ  WRTD1           ;871D: 3A 29       
		SEP  CALL              ;871F: D4          
		DW   WRITEX         ;8720: 87 A6
		DB   ';'             ;8722: 3B
		DB   0DH             ;8723: 0D
		DB   0AH             ;8724: 0A
		DB   00H             ;8725: 00
 
		SEP  CALL              ;8726: D4          
		DW   WRTH           ;8727: 87 52
WRTD1
		LDA  SRTADD              ;8729: 47          
		PHI  TMPRG3              ;872A: B9          
		SEP  CALL              ;872B: D4          
		DW   ASCII           ;872C: 87 B7
 
		BR   CKEND          ;872E: 30 0E       
FINISH
		SEP  CALL              ;8730: D4          
		DW   WRITEX           ;8731: 87 A6
		DB   CR            ;8733: 0D
		DB   EOF             ;8734: 13
		DB   00H             ;8735: 00
S8736
		LDI  80H             ;8736: F8 80       
		PLO  TMPRG3              ;8738: A9          
		LDI  0FEH            ;8739: F8 FE       
		PHI  TMPRG3              ;873B: B9          
FILBFR
		LDI  EOF             ;873C: F8 13       
		PHI  CHAR              ;873E: BF          
		SEP  CALL              ;873F: D4          
		DW   TWRITE           ;8740: 85 90                  
		INC  TMPRG3             ;8742: 19          
		GHI  TMPRG3              ;8743: 99          
		BNZ  FILBFR           ;8744: 3A 3C       
		SEP  CALL              ;8746: D4          
		DW   OSTRNG           ;8747: 83 F0
		DB   LF            ;8749: 0A
		DB   'DONE'          ;874A: 44
		                     ;874B: 4F
		                     ;874C: 4E
		                     ;874D: 45
		DB   00H             ;874E: 00
		LBR  RENTER           ;874F: C0 87 F0  
;
; ROUTINE TO WRITE A HEX NUMBER TO TAPE

WRTH
		GHI  SRTADD              ;8752: 97          
		PHI  TMPRG3              ;8753: B9          
		SEP  CALL              ;8754: D4          
		DW   ASCII           ;8755: 87 B7
 
		GLO  SRTADD             ;8757: 87          
		PHI  TMPRG3              ;8758: B9          
		SEP  CALL              ;8759: D4          
		DW   ASCII           ;875A: 87 B7
 
		SEP  CALL              ;875C: D4          
		DW   WRITEX           ;875D: 87 A6
		DB   ' '             ;875F: 20
		DB   00H             ;8760: 00
WRITER
		SEP  RETN              ;8761: D5 

		DB   0FFH            ;8762: FF
		DB   0FFH            ;8763: FF
		DB   0FFH            ;8764: FF
		DB   0FFH            ;8765: FF
		DB   0FFH            ;8766: FF
		DB   0FFH            ;8767: FF
		DB   0FFH            ;8768: FF
		DB   0FFH            ;8769: FF
		DB   0FFH            ;876A: FF
		DB   0FFH            ;876B: FF
		DB   0FFH            ;876C: FF
		DB   0FFH            ;876D: FF
		DB   0FFH            ;876E: FF
		DB   0FFH            ;876F: FF
		DB   0FFH            ;8770: FF
		DB   0FFH            ;8771: FF
		DB   0FFH            ;8772: FF
		DB   0FFH            ;8773: FF

		ORG 8774H
; ROUTINE TO GET THE TAPE DRIVE NUMBER
; AND SET UP THE IOCB

FINDTP
		SEP  CALL              ;8774: D4          
		DW   OSTRNG           ;8775: 83 F0
		DB   0DH             ;8777: 0D
		DB   0AH             ;8778: 0A
		DB   'TAPE#'         ;8779: 54

		DB   20H             ;877E: 20
		DB   00H             ;877F: 00
ASK2
		LDI  00H             ;8780: F8 00       
		PLO  ASL              ;8782: AD          
		SEP  CALL              ;8783: D4          
		DW   READCR           ;8784: 83 14
		GLO  ASL              ;8786: 8D          
		SHR                  ;8787: F6          
		BZ   FOUND           ;8788: 32 98       
		SEP  CALL              ;878A: D4          
		DW   OSTRNG           ;878B: 83 F0
		DB   0AH             ;878D: 0A
		DB   '0 OR 1?'       ;878E: 30

		DB   00H             ;8795: 00
		BR   ASK2           ;8796: 30 80 

;    FORM THE IOCB VALUES ON THE STACK

FOUND
		STXD                 ;8798: 73          
		LDI  DRIVE0             ;8799: F8 40       
		BNF  DRBIT           ;879B: 3B 9E
		SHL                  ;879D: FE          
DRBIT
		STXD                 ;879E: 73          
		LDI  00H             ;879F: F8 00       
		STXD                 ;87A1: 73          
		BR   FIXERR           ;87A2: 30 AF       
		DB   0FFH            ;87A4: FF
		DB   0FFH            ;87A5: FF

        ORG 87A6H
; TAPE WRITE SUBROUTINES
;

; WRITE IMMEDIATE

WRITEX
		LDA  LINK              ;87A6: 46          
		PHI  CHAR              ;87A7: BF          
		BZ   WRITER           ;87A8: 32 61       
		SEP  CALL              ;87AA: D4          
		DW   TWRITE           ;87AB: 85 90
		BR   WRITEX           ;87AD: 30 A6 

; REST OF THE FIND TAPE ROUTINE

FIXERR
		GHI  SP              ;87AF: 92          
		PHI  PTR              ;87B0: BC          
		GLO  SP              ;87B1: 82          
		PLO  PTR              ;87B2: AC          
		INC  PTR              ;87B3: 1C          
		INC  PTR              ;87B4: 1C          
		INC  PTR              ;87B5: 1C          
		SEP  RETN              ;87B6: D5 

;ROUTINE TO OUTPUT ASCII BYTE TO TAPE
;
ASCII	GHI  TMPRG3           ;DO THE HIGH NIBBLE
		SHR                  ;87B8: F6          
		SHR                  ;87B9: F6          
		SHR                  ;87BA: F6          
		SHR                  ;87BB: F6          
		PLO  TMPRG3              ;87BC: A9          
		SEP  CALL              ;87BD: D4          
		DW   ASCIIR           ;87BE: 87 C8

		GHI  TMPRG3              ;87C0: 99          
		ANI  0FH             ; AND THE LOW ONE
		PLO  TMPRG3              ;87C3: A9          
		SEP  CALL              ;87C4: D4          
		DW   ASCIIR           ;87C5: 87 C8
		SEP  RETN              ;87C7: D5  

ASCIIR	GLO  TMPRG3              ;87C8: 89          
		ADI  30H             ;87C9: FC 30       
		PHI  CHAR              ;87CB: BF          
		SMI  3AH             ;        
		BNF  ASC1          ;FOR 0 TO 9
		ADI  41H             ;        
		PHI  CHAR              ;FOR A TO F
ASC1
		SEP  CALL              ;87D3: D4          
		DW   TWRITE           ;87D4: 85 90
		SEP  RETN              ;87D6: D5 

; REGISTER RESTORE
; RESTORES THE REGISTERS SAVED BY REGSAV
; ASSUMES STACK POINTER I 1 BYTE ABOVE THE 
; STORED DATA. EXITS POINTING TO A FREE BYTE

REGSTR
		IRX                  ;87D7: 60          
		LDXA                 ;87D8: 72          
		PHI  R7              ;87D9: B7          
		LDXA                 ;87DA: 72          
		PLO  R7              ;87DB: A7          
		LDXA                 ;87DC: 72          
		PHI  R8              ;87DD: B8          
		LDXA                 ;87DE: 72          
		PLO  R8              ;87DF: A8          
		LDXA                 ;87E0: 72          
		PHI  R9              ;87E1: B9          
		LDXA                 ;87E2: 72          
		PLO  R9              ;87E3: A9          
		LDXA                 ;87E4: 72          
		PHI  RA              ;87E5: BA          
		LDN  SP              ;87E6: 02          
		PLO  RA              ;87E7: AA          
		GHI  RF              ;87E8: 9F          
		SEP  RETN              ;87E9: D5

 
READHX
		SEP  CALL              ;87EA: D4          
		DW   READAH          ;87EB: 81 3B
		BDF  READHX           ;87ED: 33 EA       
		SEP  RETN              ;87EF: D5 
         
   ; ENTRY POINT FOR BASIC "BYE" COMMAND AND FOR GENR REENTER
;          
        ORG	87F0H

RENTER	LDI  LOW RENTR1       ;CAN BE ENTERED WITH X
		PLO  R0              ;87F2: A0          
		LDI  HIGH RENTR1     ; AND P SET TO ANYTHING
		PHI  R0              ;87F5: B0          
		SEP  R0              ;RESETS ALL THE SCRT
RENTR1
		LDI  LOW PRMPT      ;REGISTERS
		PLO  PC              ;87F9: A3          
		LDI  HIGH PRMPT      ;87FA: F8 80       
		PHI  PC              ;87FC: B3          
		LBR  ENTER2          ;87FD: C0 83 8F    
		END
b  ADRPTR        000d  ASL           000e  AUX       
  BS            8217  BYTCNT        0004  CALL      
  CFREAD        000f  CHAR          0002  CHARAC    
  CKHEX         816f  CKHXE         000a  CNT       
  COMC2         83d4  COMCHK        002c  COMMA     
  CRLF          001d  CTLWRD        82d2  DATOUT    
  DELAY         80f3  DELAY1        80f4  DELAY2    
  DEXIT         8436  DIRECT        82bd  DISPLY    
  ENTER2        0013  EOF           8360  ERR1      
  ERROR         8363  EXITC         823f  EXITOK    
  FILL          80c5  GETDTA        83f9  GOUT62    
  HEX2          81f8  HEX3          81fb  HEX4      
  INIT1         83f6  INIT2         83a7  INSERT    
  INVERT        838f  J838F         87f0  J87F0     
  LINES         0006  LINK          000f  LNECNT    
  LOOP1         8440  MOVDN         8448  MOVDN1    
  MOVUP         82a4  MSGE          8137  NFND      
  NTDATA        0000  NULL          81de  NXCHAR    
  NXTCHR        80b7  OLDDTA        8200  OPTION    
  OUT1          82c5  OUTPUT        0003  PC        
  PRMPT         8096  PRMPT1        8246  PRMPT2    
  PROMPT        8774  PRTPN         000c  PTER      
  R0            0001  R1            0002  R2        
  R4            0005  R5            0006  R6        
  R8            8052  R8052         8085  R8085     
  R8096         80ea  R80EA         80f2  R80F2     
  R8109         810d  R810D         8117  R8117     
  R811E         812c  R812C         812f  R812F     
  R8143         8145  R8145         8149  R8149     
  R8154         8157  R8157         8159  R8159     
  R817A         81b9  R81B9         81c4  R81C4     
  R81D3         81d5  R81D5         81de  R81DE     
  R8200         8223  R8223         8246  R8246     
  R826B         8274  R8274         828a  R828A     
  R82A4         82d2  R82D2         82da  R82DA     
  R831C         831d  R831D         831f  R831F     
  R8326         832a  R832A         832c  R832C     
  R8334         833c  R833C         835d  R835D     
  R8381         83a7  R83A7         83c5  R83C5     
  R8419         841f  R841F         8428  R8428     
  R848C         84a0  R84A0         84a9  R84A9     
  R84BF         84ce  R84CE         84dc  R84DC     
  R84E7         84f9  R84F9         8579  R8579     
  R857B         85c2  R85C2         85f9  R85F9     
  R8621         8628  R8628         8637  R8637     
  R8658         8664  R8664         8667  R8667     
  R8697         869a  R869A         86a1  R86A1     
  R86C3         871a  R871A         8730  R8730     
  R8798         879e  R879E         87af  R87AF     
  R9            000a  RA            000b  RB        
  RD            000e  RE            813e  READ      
  READAH        8314  READCR        87ea  READHX    
  RETN          8374  RETR          8139  REXIT     
  ROWLEN        87d7  RREGS         8534  RUN       
  S813E         816f  S816F         82f0  S82F0     
  S83FF         8407  S8407         8474  S8474     
  S848F         8500  S8500         8502  S8502     
  S8521         852b  S852B         852e  S852E     
  S8534         8542  S8542         8557  S8557     
  S85B3         85c5  S85C5         85e7  S85E7     
  S85F1         85fe  S85FE         8601  S8601     
  S863C         8640  S8640         8648  S8648     
  S864F         8655  S8655         8659  S8659     
  S8679         869b  S869B         86a2  S86A2     
  S86D4         86df  S86DF         86e9  S86E9     
  S86FC         8704  S8704         870b  S870B     
  S8726         8729  S8729         872e  S872E     
  S8742         874f  S874F         8752  S8752     
  S875C         8761  S8761         8774  S8774     
  S8786         87a6  S87A6         87ad  S87AD     
  S87C0         87c7  S87C7         87c8  S87C8     
  S87D7         87ea  S87EA         87ed  S87ED     
  SAMELN        804f  SCAN          8043  SCAN1     
  SP            0020  SPACE         82cd  SPCOUT    
  SREGS         802c  START         0003  STATUS    
  TAB2          819f  TEXIT         80fe  TIMALC    
  TMPRG2        0009  TMPRGS        8cff  TOPSTK    
  TPTR          0002  TRMINL        81b9  TY1       
  TY3           81a4  TYPE          81ae  TYPE2     
  TYPE5D        81a2  TYPE6         8198  TYPED     
  UP            846d  UP1           0003  URTCTL    
  USRFIL        8429  USRMOV        8000  UT62      
  WRAM          
